/*
 *  Copyright 2019 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.plugins.odfweb;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;

import org.ametys.cms.content.indexing.solr.observation.ObserverHelper;
import org.ametys.core.right.Profile;
import org.ametys.core.right.ProfileAssignmentStorageExtensionPoint;
import org.ametys.core.right.RightProfilesDAO;
import org.ametys.core.user.CurrentUserProvider;
import org.ametys.odf.ODFContentHelper;
import org.ametys.odf.ODFHelper;
import org.ametys.odf.catalog.CatalogDAO;
import org.ametys.odf.enumeration.OdfReferenceTableHelper;
import org.ametys.plugins.repository.AmetysObjectResolver;
import org.ametys.runtime.test.framework.AbstractPopulationAwareTestCase;
import org.ametys.runtime.test.framework.CocoonTestCase.NeedFeatures;
import org.ametys.web.repository.site.SiteManager;
import org.ametys.web.test.framework.WithWebFeaturesTestCase;

/**
 * Abstract test case for all Web test cases.
 * This class enables to create sites and sitemaps from a XML file.
 * TODO This class should be moved into Web tests
 */
@NeedFeatures({
    // Direct dependencies (components)
    "repository/resolver",
    "web/sites",
    "odf-web/odfweb.catalog.dao",
    "cms/workflow",
    "odf/odf.ref-table.helper",
    "core/runtime.rights.profiles.dao",
    "test/test.user.provider",
    "test/test.odf.content.helper",
    "odf/workflow.sendcdmfr.onvalidation",
    "odf/workflow.depositcdmfr.onvalidation",
    "odf-web/workflow.program",
    "odf/workflow.subprogram",
    "odf/workflow.container",
    "odf/workflow.courselist",
    "odf-web/workflow.course",
    "odf/workflow.course.part",
    "odf-web/workflow.orgunit",
    "odf/workflow.person",
    "odf/odf.helper",
    
    // For populating reference tables
    "odf-web/init.table-ref",
    
    // for RightManager#hasRight
    "odf/org.ametys.odf.content.right.accesscontroller",
    "repository/repository.rights.storage.assignments.impl.jcr",
    
    // Creation of objects
    "web/sitemap-service",
    "web/default.content-type.article",
    "odf/odf.content-types.program",
    "odf/odf.content-types.subprogram",
    "odf/odf.content-types.container",
    "odf/odf.content-types.courselist",
    "odf/odf.content-types.course",
    "cms/workflow.reference.table",
    
    // For enumerators,
    "web/default-services-handler"
})
public abstract class AbstractOdfWebTestCase extends AbstractPopulationAwareTestCase implements WithWebFeaturesTestCase
{
    /** Domain */
    protected static final String __DOMAIN_ALL = "ALL";
    /** Degree A */
    protected static final String __DEGREE_A = "XA";
    /** Degree B */
    protected static final String __DEGREE_B = "XB";
    /** Period year */
    protected static final String __PERIOD_YEAR = "an";
    /** Period semester 1 */
    protected static final String __PERIOD_SEM_1 = "s1";
    /** Period semester 2 */
    protected static final String __PERIOD_SEM_2 = "s2";
    /** Container nature year */
    protected static final String __CONTAINER_NATURE_YEAR = "annee";
    /** Container nature semester */
    protected static final String __CONTAINER_NATURE_SEM = "semestre";
    /** Course nature UE */
    protected static final String __COURSE_NATURE_UE = "UE";
    /** Course nature EC */
    protected static final String __COURSE_NATURE_EC = "EC";
    
    /** Ametys object resolver to subclasses. */
    protected static AmetysObjectResolver _ametysObjectResolver;
    /** The site manager */
    protected static SiteManager _siteManager;
    /** Catalogs manager */
    protected static CatalogDAO _catalogDAO;
    /** ODF Reference table helper */
    protected static OdfReferenceTableHelper _odfReferenceTableHelper;
    /** The profiles DAO */
    protected static RightProfilesDAO _profilesDAO;
    /** The profile assignment */
    protected static ProfileAssignmentStorageExtensionPoint _profileAssignmentStorageEP;
    /** The current user provider */
    protected static CurrentUserProvider _currentUserProvider;
    /** Helper to create a complete ODF structure for test cases */
    protected static ODFContentHelper _odfContentHelper;
    /** The ODF helpers */
    protected static ODFHelper _odfHelper;

    private static Profile _profile;
    
    public File[] getStartScriptFiles()
    {
        return new File[0];
    }
    
    @Override
    public Map<String, String> variableValues()
    {
        Map<String, String> values = new HashMap<>(super.variableValues());
        values.put("solrPort", "65535");
        return values;
    }
    
    @Override
    protected void essentialsBefore() throws Exception
    {
        ObserverHelper.suspendObservationForIndexation();
        super.essentialsBefore();
    }
    
    @Override
    public void lookupComponents(ServiceManager manager) throws ServiceException
    {
        super.lookupComponents(manager);
        _ametysObjectResolver = (AmetysObjectResolver) manager.lookup(AmetysObjectResolver.ROLE);
        _siteManager = (SiteManager) manager.lookup(SiteManager.ROLE);
        _catalogDAO = (CatalogDAO) manager.lookup(CatalogDAO.ROLE);
        _odfReferenceTableHelper = (OdfReferenceTableHelper) manager.lookup(OdfReferenceTableHelper.ROLE);
        _profilesDAO = (RightProfilesDAO) manager.lookup(RightProfilesDAO.ROLE);
        _profileAssignmentStorageEP = (ProfileAssignmentStorageExtensionPoint) manager.lookup(ProfileAssignmentStorageExtensionPoint.ROLE);
        _currentUserProvider = (CurrentUserProvider) manager.lookup(CurrentUserProvider.ROLE);
        _odfContentHelper = (ODFContentHelper) manager.lookup(ODFContentHelper.ROLE);
        _odfHelper = (ODFHelper) manager.lookup(ODFHelper.ROLE);
    }
    
    @Override
    public void prepareComponents() throws Exception
    {
        super.prepareComponents();
        _assignRightsToUser();
    }

    private void _assignRightsToUser()
    {
        _profile = _profilesDAO.addProfile("profile-test");

        List<String> rightsNeeded = new ArrayList<>();
        rightsNeeded.add("ODF_Rights_Program_Synchro");
        rightsNeeded.add("ODF_Rights_SubProgram_Synchro");
        rightsNeeded.add("ODF_Rights_Container_Synchro");
        rightsNeeded.add("ODF_Rights_CourseList_Synchro");
        rightsNeeded.add("ODF_Rights_Course_Synchro");
        rightsNeeded.add("ODF_Rights_OrgUnit_Synchro");
        rightsNeeded.add("ODF_Rights_Person_Synchro");
        rightsNeeded.add("ODF_Rights_Strategics_Fields");
        rightsNeeded.add("ODF_Rights_Program_Validate");
        rightsNeeded.add("ODF_Rights_Course_Validate");
        rightsNeeded.add("ODF_Rights_Person_Edit");
        _profilesDAO.addRights(_profile, rightsNeeded);

        _profileAssignmentStorageEP.allowProfileToUser(CURRENT_USER_TEST_IDENTITY, _profile.getId(), "/test");
        _profileAssignmentStorageEP.allowProfileToUser(CURRENT_USER_TEST_IDENTITY, _profile.getId(), "/cms");
        _profileAssignmentStorageEP.allowProfileToUser(CURRENT_USER_TEST_IDENTITY, _profile.getId(), _odfHelper.getRootContent());
    }
    
    @Override
    protected void essentialsAfter() throws Exception
    {
        _removeProfile();
        super.essentialsAfter();
    }
    
    private void _removeProfile()
    {
        if (_profile != null)
        {
            _profileAssignmentStorageEP.removeAllowedProfileFromUser(CURRENT_USER_TEST_IDENTITY, _profile.getId(), "/test");
            _profileAssignmentStorageEP.removeAllowedProfileFromUser(CURRENT_USER_TEST_IDENTITY, _profile.getId(), "/cms");
            _profileAssignmentStorageEP.removeAllowedProfileFromUser(CURRENT_USER_TEST_IDENTITY, _profile.getId(), _odfHelper.getRootContent());
            _profilesDAO.deleteProfile(_profile);
        }
    }
    
    /**
     * Configures sites and sitemaps from the given file
     * @param fileName the name of the sitemap configuration file
     * @throws Exception if the sitemaps cannot be created
     */
    protected void _configureSitemaps(String fileName) throws Exception
    {
        SitemapConfigurator sitemapConfigurator = new SitemapConfigurator(_ametysObjectResolver, _siteManager, CURRENT_USER_TEST_IDENTITY);
        sitemapConfigurator._configureSitemaps(fileName);
    }
}
