/*
 *  Copyright 2017 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.plugins.odfweb;

import java.util.List;
import java.util.Map;

import javax.jcr.Node;
import javax.jcr.PathNotFoundException;
import javax.jcr.Repository;
import javax.jcr.RepositoryException;
import javax.jcr.ValueFormatException;

import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.junit.After;
import org.junit.Assert;
import org.junit.Test;
import org.quartz.SchedulerException;
import org.quartz.TriggerKey;

import org.ametys.cms.repository.Content;
import org.ametys.odf.ProgramItem;
import org.ametys.odf.catalog.Catalog;
import org.ametys.odf.catalog.CatalogsManager;
import org.ametys.odf.course.Course;
import org.ametys.odf.course.CourseFactory;
import org.ametys.odf.courselist.CourseList;
import org.ametys.odf.courselist.CourseList.ChoiceType;
import org.ametys.odf.courselist.CourseListFactory;
import org.ametys.odf.enumeration.OdfReferenceTableHelper;
import org.ametys.odf.orgunit.OrgUnit;
import org.ametys.odf.orgunit.OrgUnitFactory;
import org.ametys.odf.person.PersonFactory;
import org.ametys.odf.program.AbstractProgram;
import org.ametys.odf.program.Container;
import org.ametys.odf.program.ContainerFactory;
import org.ametys.odf.program.Program;
import org.ametys.odf.program.ProgramFactory;
import org.ametys.odf.program.SubProgram;
import org.ametys.odf.program.SubProgramFactory;
import org.ametys.plugins.core.schedule.Scheduler;
import org.ametys.plugins.repository.AmetysObjectResolver;
import org.ametys.runtime.test.framework.CocoonTestCase.NeedFeatures;

/**
 * Test case for ODF base operation 
 */
@NeedFeatures({
    // Direct dependencies (components)
    "repository/provider.jackrabbit",
    "core/model.helper",
    "repository/repository.data.holder.helpers",
    "odf/odf-catalog-manager",
    "core/runtime.core.scheduler",
    "workflow/condition.current.step"
})
public class ODFBaseOperationTestCase extends AbstractOdfWebTestCase
{
    /** The repository */
    protected static Repository _repository;
    private static CatalogsManager _catalogManager;
    private static Scheduler _scheduler;
    
    @Override
    public void lookupComponents(ServiceManager manager) throws ServiceException
    {
        super.lookupComponents(manager);
        _repository = (Repository) manager.lookup(Repository.class.getName());
        _catalogManager = (CatalogsManager) manager.lookup(CatalogsManager.ROLE);
        _scheduler = (Scheduler) manager.lookup(Scheduler.ROLE);
    }
    
    /**
     * Removes the existing catalogs after each test
     */
    @After
    public void removeCatalogs()
    {
        for (Catalog catalog : _catalogManager.getCatalogs())
        {
            String id = catalog.getId();
            _catalogManager.deleteCatalog(catalog);
            _logger.info("Catalog '{}' was removed", id);
        }
    }
    
    /**
     * Retrieves the ametys root node.
     * @return the JCR node.
     * @throws RepositoryException if an error occurs.
     */
    private Node _getAmetysRootNode() throws RepositoryException
    {
        return _repository.login().getRootNode().getNode(AmetysObjectResolver.ROOT_REPO);
    }

    /**
     * Tester les opérations sur les catalogues via CatalogDAO
     * @throws Exception if an error occurs
     */
    @Test
    public void testCatalogDAO() throws Exception
    {
        Node ametysRootNode = _getAmetysRootNode();
        
        String catalogue1 = "catalogue1";
        String catalogue2 = "catalogue2";
        
        Map<String, String> catalogue1Result = _catalogDAO.createCatalog("Titre " + catalogue1, catalogue1, null);
        Node catalogsNode = ametysRootNode.getNode("ametys:plugins/odf/catalogs");
        Assert.assertTrue(catalogsNode.hasNode(catalogue1));
        Node catalog1Node = catalogsNode.getNode(catalogue1);
        Assert.assertTrue(catalog1Node.hasProperty("ametys:isDefault"));
        Assert.assertTrue(catalog1Node.getProperty("ametys:isDefault").getBoolean());
        
        Map<String, String> catalogue2Result = _catalogDAO.createCatalog("Titre " + catalogue2, catalogue2, catalogue1);
        Assert.assertTrue(catalogsNode.hasNode(catalogue2));
        Node catalog2Node = catalogsNode.getNode(catalogue2);
        Assert.assertTrue(!catalog2Node.hasProperty("ametys:isDefault") || !catalog2Node.getProperty("ametys:isDefault").getBoolean());
        
        _catalogDAO.editCatalog(catalogue2Result.get("id"), "Second catalogue");
        Assert.assertTrue(catalogsNode.hasNode(catalogue2));
        catalog2Node = catalogsNode.getNode(catalogue2);
        Assert.assertEquals("Second catalogue", catalog2Node.getProperty("ametys:title").getString());
        
        _catalogDAO.setDefaultCatalog(catalogue2Result.get("id"));
        catalog1Node = catalogsNode.getNode(catalogue1);
        catalog2Node = catalogsNode.getNode(catalogue2);
        Assert.assertTrue(!catalog1Node.hasProperty("ametys:isDefault") || !catalog1Node.getProperty("ametys:isDefault").getBoolean());
        Assert.assertTrue(catalog2Node.getProperty("ametys:isDefault").getBoolean());

        Map<String, Object> result = _catalogDAO.removeCatalog(catalogue2Result.get("id"), true);
        waitForExecutingJob((String) result.get("jobId"));
        Assert.assertFalse(catalogsNode.hasNode(catalogue2));
        Assert.assertTrue(catalogsNode.hasNode(catalogue1));
        
        Map<String, Object> properties = _catalogDAO.getCatalogProperties(catalogue1Result.get("id"));
        Assert.assertTrue(properties.containsKey("id"));
        Assert.assertEquals(catalogue1Result.get("id"), properties.get("id"));
        Assert.assertTrue(properties.containsKey("isDefault"));
        Assert.assertFalse((Boolean) properties.get("isDefault"));
        Assert.assertTrue(properties.containsKey("title"));
        Assert.assertEquals("Titre " + catalogue1, properties.get("title"));
        Assert.assertTrue(properties.containsKey("code"));
        Assert.assertEquals(catalogue1, properties.get("code"));
        Assert.assertTrue(properties.containsKey("nbPrograms"));
    }
    
    private void _testInternalValues(Node contentNode, String contentType, String primaryType) throws ValueFormatException, PathNotFoundException, RepositoryException
    {
        Assert.assertTrue(contentNode.hasProperty("jcr:primaryType"));
        Assert.assertEquals(primaryType, contentNode.getProperty("jcr:primaryType").getString());
        
        Assert.assertTrue(contentNode.hasProperty("ametys-internal:workflowId"));
        
        Assert.assertTrue(contentNode.hasProperty("ametys-internal:currentStepId"));
        Assert.assertEquals(1, contentNode.getProperty("ametys-internal:currentStepId").getLong());
        
        Assert.assertTrue(contentNode.hasProperty("ametys-internal:language"));
        Assert.assertEquals("fr", contentNode.getProperty("ametys-internal:language").getString());
        
        Assert.assertTrue(contentNode.hasProperty("ametys-internal:contentType"));
        Assert.assertEquals(1, contentNode.getProperty("ametys-internal:contentType").getValues().length);
        Assert.assertEquals(contentType, contentNode.getProperty("ametys-internal:contentType").getValues()[0].getString());

        Assert.assertTrue(contentNode.hasProperty("ametys:creationDate"));
        Assert.assertTrue(contentNode.hasProperty("ametys:lastModified"));
    }
    
    private void waitForExecutingJob(String jobId) throws SchedulerException
    {
        while (_scheduler.getScheduler().checkExists(new TriggerKey(jobId, Scheduler.TRIGGER_GROUP)))
        {
            try
            {
                Thread.sleep(1000);
            }
            catch (InterruptedException e)
            {
                // TODO
            }
        }
    }

    /**
     * Tester la création des contenus ODF via l'API
     * @throws Exception if an error occurs
     */
    @Test
    public void testODFContentCreation() throws Exception
    {
        Node ametysRootNode = _getAmetysRootNode();
        
        // Create Catalog
        String catalogName = "2017";
        _catalogDAO.createCatalog("Catalogue 2017", catalogName, null);
        // Tested through testCatalogsDAO

        // Create Person
        String title = "Personne TU 1";
        Content person = _odfContentHelper.new PersonBuilder()
                .fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, title))
                .build();

        // Récupération noeud racine des contenus ODF
        Node contentsNode = ametysRootNode.getNode("ametys:plugins/odf/ametys:contents");
        
        // Tests Person
        Assert.assertTrue(contentsNode.hasNode("48/91/person-personne-tu-1"));
        Node contentNode = contentsNode.getNode("48/91/person-personne-tu-1");
        _testInternalValues(contentNode, PersonFactory.PERSON_CONTENT_TYPE, PersonFactory.PERSON_NODETYPE);
        Assert.assertTrue(contentNode.hasProperty("ametys:title"));
        Assert.assertEquals(title, contentNode.getProperty("ametys:title").getString());
        Assert.assertTrue(contentNode.hasProperty("ametys:lastName"));
        Assert.assertEquals(title, contentNode.getProperty("ametys:lastName").getString());
        
        // Create OrgUnit
        title = "Composante TU 1";
        OrgUnit orgUnit = _odfContentHelper.new OrgUnitBuilder()
                .fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, title,
                        OrgUnit.CODE_UAI, "UAITU1",
                        OrgUnit.CONTACTS, List.of(person)))
                .build();
        
        // Tests OrgUnit
        Assert.assertTrue(contentsNode.hasNode("56/ac/orgunit-composante-tu-1"));
        contentNode = contentsNode.getNode("56/ac/orgunit-composante-tu-1");
        _testInternalValues(contentNode, OrgUnitFactory.ORGUNIT_CONTENT_TYPE, OrgUnitFactory.ORGUNIT_NODETYPE);
        Assert.assertTrue(contentNode.hasProperty("ametys:title"));
        Assert.assertEquals(title, contentNode.getProperty("ametys:title").getString());
        Assert.assertTrue(contentNode.hasProperty("ametys:codeUAI"));
        Assert.assertEquals("UAITU1", contentNode.getProperty("ametys:codeUAI").getString());
        Assert.assertTrue(contentNode.hasProperty("ametys:contactsReferences"));
        Assert.assertEquals(1, contentNode.getProperty("ametys:contactsReferences").getValues().length);
        Assert.assertEquals(person.getId(), contentNode.getProperty("ametys:contactsReferences").getValues()[0].getString());

        // Create Program
        title = "Formation TU 1";
        _odfContentHelper.new ProgramBuilder()
            .withDefaultCatalog(catalogName)
            .withDefaultOrgUnits(orgUnit)
            .fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, title,
                    "contacts", List.of(Map.of("persons", List.of(person)))))
            .build();
        
        // Tests Program
        Assert.assertTrue(contentsNode.hasNode("62/66/program-formation-tu-1"));
        contentNode = contentsNode.getNode("62/66/program-formation-tu-1");
        _testInternalValues(contentNode, ProgramFactory.PROGRAM_CONTENT_TYPE, ProgramFactory.PROGRAM_NODETYPE);
        Assert.assertTrue(contentNode.hasProperty("ametys:title"));
        Assert.assertEquals(title, contentNode.getProperty("ametys:title").getString());
        Assert.assertTrue(contentNode.hasProperty("ametys:catalog"));
        Assert.assertEquals(catalogName, contentNode.getProperty("ametys:catalog").getString());
        Assert.assertTrue(contentNode.hasProperty("ametys:domain"));
        String domainId = _odfReferenceTableHelper.getItemFromCode(OdfReferenceTableHelper.DOMAIN, __DOMAIN_ALL).getId();
        Assert.assertEquals(domainId, contentNode.getProperty("ametys:domain").getValues()[0].getString());
        Assert.assertTrue(contentNode.hasProperty("ametys:degree"));
        String degreeId = _odfReferenceTableHelper.getItemFromCode(OdfReferenceTableHelper.DEGREE, __DEGREE_A).getId();
        Assert.assertEquals(degreeId, contentNode.getProperty("ametys:degree").getString());
        Assert.assertTrue(contentNode.hasProperty("ametys:orgUnit"));
        Assert.assertEquals(1, contentNode.getProperty("ametys:orgUnit").getValues().length);
        Assert.assertEquals(orgUnit.getId(), contentNode.getProperty("ametys:orgUnit").getValues()[0].getString());
        Assert.assertTrue(contentNode.hasNode("ametys:contacts"));
        Assert.assertTrue(contentNode.getNode("ametys:contacts").hasNode("ametys:1"));
        Assert.assertTrue(contentNode.getNode("ametys:contacts").getNode("ametys:1").hasProperty("ametys:persons"));
        Assert.assertEquals(1, contentNode.getNode("ametys:contacts").getNode("ametys:1").getProperty("ametys:persons").getValues().length);
        Assert.assertEquals(person.getId(), contentNode.getNode("ametys:contacts").getNode("ametys:1").getProperty("ametys:persons").getValues()[0].getString());
        
        // Create SubProgram
        title = "Parcours TU 1";
        _odfContentHelper.new SubProgramBuilder()
            .withDefaultCatalog(catalogName)
            .withDefaultOrgUnits(orgUnit)
            .fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, title,
                    "contacts", List.of(Map.of("persons", List.of(person)))))
            .build();

        // Tests SubProgram
        Assert.assertTrue(contentsNode.hasNode("e2/54/subprogram-parcours-tu-1"));
        contentNode = contentsNode.getNode("e2/54/subprogram-parcours-tu-1");
        _testInternalValues(contentNode, SubProgramFactory.SUBPROGRAM_CONTENT_TYPE, SubProgramFactory.SUBPROGRAM_NODETYPE);
        Assert.assertTrue(contentNode.hasProperty("ametys:title"));
        Assert.assertEquals(title, contentNode.getProperty("ametys:title").getString());
        Assert.assertTrue(contentNode.hasProperty("ametys:catalog"));
        Assert.assertEquals(catalogName, contentNode.getProperty("ametys:catalog").getString());
        Assert.assertTrue(contentNode.hasProperty("ametys:orgUnit"));
        Assert.assertEquals(1, contentNode.getProperty("ametys:orgUnit").getValues().length);
        Assert.assertEquals(orgUnit.getId(), contentNode.getProperty("ametys:orgUnit").getValues()[0].getString());
        Assert.assertTrue(contentNode.hasNode("ametys:contacts"));
        Assert.assertTrue(contentNode.getNode("ametys:contacts").hasNode("ametys:1"));
        Assert.assertTrue(contentNode.getNode("ametys:contacts").getNode("ametys:1").hasProperty("ametys:persons"));
        Assert.assertEquals(1, contentNode.getNode("ametys:contacts").getNode("ametys:1").getProperty("ametys:persons").getValues().length);
        Assert.assertEquals(person.getId(), contentNode.getNode("ametys:contacts").getNode("ametys:1").getProperty("ametys:persons").getValues()[0].getString());

        // Create Container
        title = "Conteneur TU 1";
        _odfContentHelper.new ContainerBuilder()
            .withDefaultCatalog(catalogName)
            .fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, title))
            .build();

        // Tests Container
        Assert.assertTrue(contentsNode.hasNode("79/39/container-conteneur-tu-1"));
        contentNode = contentsNode.getNode("79/39/container-conteneur-tu-1");
        _testInternalValues(contentNode, ContainerFactory.CONTAINER_CONTENT_TYPE, ContainerFactory.CONTAINER_NODETYPE);
        Assert.assertTrue(contentNode.hasProperty("ametys:title"));
        Assert.assertEquals(title, contentNode.getProperty("ametys:title").getString());
        Assert.assertTrue(contentNode.hasProperty("ametys:catalog"));
        Assert.assertEquals(catalogName, contentNode.getProperty("ametys:catalog").getString());
        
        // Create CourseList
        title = "Liste TU 1";
        _odfContentHelper.new CourseListBuilder()
            .withDefaultCatalog(catalogName)
            .fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, title,
                    CourseList.CHOICE_TYPE, ChoiceType.OPTIONAL))
            .build();

        // Tests CourseList
        Assert.assertTrue(contentsNode.hasNode("55/ee/courselist-liste-tu-1"));
        contentNode = contentsNode.getNode("55/ee/courselist-liste-tu-1");
        _testInternalValues(contentNode, CourseListFactory.COURSE_LIST_CONTENT_TYPE, CourseListFactory.COURSE_LIST_NODETYPE);
        Assert.assertTrue(contentNode.hasProperty("ametys:title"));
        Assert.assertEquals(title, contentNode.getProperty("ametys:title").getString());
        Assert.assertTrue(contentNode.hasProperty("ametys:catalog"));
        Assert.assertEquals(catalogName, contentNode.getProperty("ametys:catalog").getString());
        Assert.assertTrue(contentNode.hasProperty("ametys:choiceType"));
        Assert.assertEquals("OPTIONAL", contentNode.getProperty("ametys:choiceType").getString());

        // Create Course
        title = "ELP TU 1";
        _odfContentHelper.new CourseBuilder()
            .withDefaultCatalog(catalogName)
            .withDefaultOrgUnits(orgUnit)
            .fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, title,
                    "contacts", List.of(Map.of("persons", List.of(person)))))
            .build();

        // Tests Course
        Assert.assertTrue(contentsNode.hasNode("69/a7/course-elp-tu-1"));
        contentNode = contentsNode.getNode("69/a7/course-elp-tu-1");
        _testInternalValues(contentNode, CourseFactory.COURSE_CONTENT_TYPE, CourseFactory.COURSE_NODETYPE);
        Assert.assertTrue(contentNode.hasProperty("ametys:title"));
        Assert.assertEquals(title, contentNode.getProperty("ametys:title").getString());
        Assert.assertTrue(contentNode.hasProperty("ametys:catalog"));
        Assert.assertEquals(catalogName, contentNode.getProperty("ametys:catalog").getString());
        Assert.assertTrue(contentNode.hasProperty("ametys:orgUnit"));
        Assert.assertEquals(1, contentNode.getProperty("ametys:orgUnit").getValues().length);
        Assert.assertEquals(orgUnit.getId(), contentNode.getProperty("ametys:orgUnit").getValues()[0].getString());
        Assert.assertTrue(contentNode.hasNode("ametys:contacts"));
        Assert.assertTrue(contentNode.getNode("ametys:contacts").hasNode("ametys:1"));
        Assert.assertTrue(contentNode.getNode("ametys:contacts").getNode("ametys:1").hasProperty("ametys:persons"));
        Assert.assertEquals(1, contentNode.getNode("ametys:contacts").getNode("ametys:1").getProperty("ametys:persons").getValues().length);
        Assert.assertEquals(person.getId(), contentNode.getNode("ametys:contacts").getNode("ametys:1").getProperty("ametys:persons").getValues()[0].getString());
    }

    /**
     * Tester les liens entre les contenus ODF créés via l'API
     * @throws Exception if an error occurs
     */
    @Test
    public void testODFContentLinks() throws Exception
    {
        // Create Catalog
        String catalogName = "2017";
        _catalogDAO.createCatalog("Catalogue 2017", catalogName, null);
        
        // Create OrgUnits
        /*
         * OU1
         *  | OU2
         *     | OU3
         *     | OU4
         *  | OU5
         */
        OrgUnit orgUnit5 = _odfContentHelper.new OrgUnitBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "Composante TU 5", OrgUnit.CODE_UAI, "UAITU1")).build();
        OrgUnit orgUnit4 = _odfContentHelper.new OrgUnitBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "Composante TU 4", OrgUnit.CODE_UAI, "UAITU1")).build();
        OrgUnit orgUnit3 = _odfContentHelper.new OrgUnitBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "Composante TU 3", OrgUnit.CODE_UAI, "UAITU1")).build();
        OrgUnit orgUnit2 = _odfContentHelper.new OrgUnitBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "Composante TU 2", OrgUnit.CODE_UAI, "UAITU1", OrgUnit.CHILD_ORGUNITS, List.of(orgUnit3, orgUnit4))).build();
        OrgUnit orgUnit1 = _odfContentHelper.new OrgUnitBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "Composante TU 1", OrgUnit.CODE_UAI, "UAITU1", OrgUnit.CHILD_ORGUNITS, List.of(orgUnit2, orgUnit5))).build();
        
        // Create Program
        Program program1 = _odfContentHelper.new ProgramBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "Formation TU 1", ProgramItem.ORG_UNITS_REFERENCES, List.of(orgUnit1))).withDefaultCatalog(catalogName).build();
        Program program2 = _odfContentHelper.new ProgramBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "Formation TU 2", ProgramItem.ORG_UNITS_REFERENCES, List.of(orgUnit1))).withDefaultCatalog(catalogName).build();
        Program program3 = _odfContentHelper.new ProgramBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "Formation TU 3", ProgramItem.ORG_UNITS_REFERENCES, List.of(orgUnit1), AbstractProgram.DEGREE, __DEGREE_B)).withDefaultCatalog(catalogName).build();
        
        // Create SubProgram
        SubProgram subProgram1 = _odfContentHelper.new SubProgramBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "Parcours TU 1", ProgramItem.ORG_UNITS_REFERENCES, List.of(orgUnit1))).withDefaultCatalog(catalogName).build();
        SubProgram subProgram2 = _odfContentHelper.new SubProgramBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "Parcours TU 2", ProgramItem.ORG_UNITS_REFERENCES, List.of(orgUnit1))).withDefaultCatalog(catalogName).build();
        SubProgram subProgram3 = _odfContentHelper.new SubProgramBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "Parcours TU 3", ProgramItem.ORG_UNITS_REFERENCES, List.of(orgUnit1))).withDefaultCatalog(catalogName).build();

        // Create Container
        Container container1 = _odfContentHelper.new ContainerBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "Conteneur TU 1")).build();
        Container container2 = _odfContentHelper.new ContainerBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "Conteneur TU 2", Container.PERIOD, __PERIOD_SEM_1, Container.NATURE, __CONTAINER_NATURE_SEM)).withDefaultCatalog(catalogName).build();
        Container container3 = _odfContentHelper.new ContainerBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "Conteneur TU 3", Container.PERIOD, __PERIOD_SEM_1, Container.NATURE, __CONTAINER_NATURE_SEM)).withDefaultCatalog(catalogName).build();
        Container container4 = _odfContentHelper.new ContainerBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "Conteneur TU 4", Container.PERIOD, __PERIOD_SEM_2, Container.NATURE, __CONTAINER_NATURE_SEM)).withDefaultCatalog(catalogName).build();
        Container container5 = _odfContentHelper.new ContainerBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "Conteneur TU 5", Container.PERIOD, __PERIOD_SEM_2, Container.NATURE, __CONTAINER_NATURE_SEM)).withDefaultCatalog(catalogName).build();
        
        // Create CourseList
        CourseList courseList1 = _odfContentHelper.new CourseListBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "Liste TU 1", CourseList.CHOICE_TYPE, ChoiceType.OPTIONAL)).withDefaultCatalog(catalogName).build();
        CourseList courseList2 = _odfContentHelper.new CourseListBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "Liste TU 2")).withDefaultCatalog(catalogName).build();
        CourseList courseList3 = _odfContentHelper.new CourseListBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "Liste TU 3")).withDefaultCatalog(catalogName).build();
        CourseList courseList4 = _odfContentHelper.new CourseListBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "Liste TU 4")).withDefaultCatalog(catalogName).build();

        // Create Course
        Course course1 = _odfContentHelper.new CourseBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "ELP TU 1", Course.COURSE_TYPE, __COURSE_NATURE_EC)).withDefaultCatalog(catalogName).build();
        Course course2 = _odfContentHelper.new CourseBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "ELP TU 2", Course.COURSE_TYPE, __COURSE_NATURE_EC)).withDefaultCatalog(catalogName).build();
        Course course3 = _odfContentHelper.new CourseBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "ELP TU 3", Course.COURSE_TYPE, __COURSE_NATURE_EC)).withDefaultCatalog(catalogName).build();
        Course course4 = _odfContentHelper.new CourseBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "ELP TU 4", Course.COURSE_TYPE, __COURSE_NATURE_EC)).withDefaultCatalog(catalogName).build();
        Course course5 = _odfContentHelper.new CourseBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "ELP TU 5")).withDefaultCatalog(catalogName).build();
        Course course6 = _odfContentHelper.new CourseBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "ELP TU 6")).withDefaultCatalog(catalogName).build();
        Course course7 = _odfContentHelper.new CourseBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "ELP TU 7")).withDefaultCatalog(catalogName).build();
        
        /* CL1
         *  | C1
         *  | C2
         *  
         * CL2
         *  | C1
         *  | C3
         *  | C4
         * 
         * CL3
         *  | C5
         *  | C6
         * 
         * CL4
         *  | C3
         *  | C7
         */
        _odfContentHelper.editContent(courseList1, Map.of("courses", List.of(course1, course2)));
        _odfContentHelper.editContent(courseList2, Map.of("courses", List.of(course1, course3, course4)));
        _odfContentHelper.editContent(courseList3, Map.of("courses", List.of(course5, course6)));
        _odfContentHelper.editContent(courseList4, Map.of("courses", List.of(course3, course7)));
        
        /* 
         * C1
         *  | CL3
         * 
         * C2
         *  | CL3
         *  | CL4
         */
        _odfContentHelper.editContent(course1, Map.of("courseLists", List.of(courseList3)));
        _odfContentHelper.editContent(course2, Map.of("courseLists", List.of(courseList3, courseList4)));
        
        /*
         * P1
         *  | SP1
         *  | SP2
         *  
         * P2
         *  | CT1
         *  | CT2
         *  
         * P3
         *  | SP1
         *  | CT1
         *  | CL1
         *  | SP3
         */
        _odfContentHelper.editContent(program1, Map.of("childProgramParts", List.of(subProgram1, subProgram2)));
        _odfContentHelper.editContent(program2, Map.of("childProgramParts", List.of(container1, container2)));
        _odfContentHelper.editContent(program3, Map.of("childProgramParts", List.of(subProgram1, container1, courseList1, subProgram3)));
        
        /*
         * SP1
         *  | CT3
         *  | CT4
         *  | CT1
         * 
         * SP2
         *  | CT3
         *  | SP3
         * 
         * SP3
         *  | CL2
         *  | CL3
         *  | CL4
         */
        _odfContentHelper.editContent(subProgram1, Map.of("childProgramParts", List.of(container3, container4.getId(), container1)));
        _odfContentHelper.editContent(subProgram2, Map.of("childProgramParts", List.of(container3, subProgram3)));
        _odfContentHelper.editContent(subProgram3, Map.of("childProgramParts", List.of(courseList2, courseList3, courseList4)));
        
        /*
         * CT1
         *  | CL2
         *  | CT5
         *  
         * CT2
         *  | SP3
         */
        _odfContentHelper.editContent(container1, Map.of("childProgramParts", List.of(courseList2, container5)));
        _odfContentHelper.editContent(container2, Map.of("childProgramParts", List.of(subProgram3)));

        // orgUnit1
        Node orgUnit1Node = orgUnit1.getNode();
        Assert.assertTrue(orgUnit1Node.hasProperty("ametys:childOrgUnits"));
        Assert.assertEquals(2, orgUnit1Node.getProperty("ametys:childOrgUnits").getValues().length);
        Assert.assertEquals(orgUnit2.getId(), orgUnit1Node.getProperty("ametys:childOrgUnits").getValues()[0].getString());
        Assert.assertEquals(orgUnit5.getId(), orgUnit1Node.getProperty("ametys:childOrgUnits").getValues()[1].getString());
        
        // orgUnit2
        Node orgUnit2Node = orgUnit2.getNode();
        Assert.assertTrue(orgUnit2Node.hasProperty("ametys:childOrgUnits"));
        Assert.assertEquals(2, orgUnit2Node.getProperty("ametys:childOrgUnits").getValues().length);
        Assert.assertEquals(orgUnit3.getId(), orgUnit2Node.getProperty("ametys:childOrgUnits").getValues()[0].getString());
        Assert.assertEquals(orgUnit4.getId(), orgUnit2Node.getProperty("ametys:childOrgUnits").getValues()[1].getString());
        Assert.assertTrue(orgUnit2Node.hasProperty("ametys:parentOrgUnit"));
        Assert.assertEquals(orgUnit1.getId(), orgUnit2Node.getProperty("ametys:parentOrgUnit").getString());
        
        // orgUnit3
        Node orgUnit3Node = orgUnit3.getNode();
        Assert.assertTrue(orgUnit3Node.hasProperty("ametys:parentOrgUnit"));
        Assert.assertEquals(orgUnit2.getId(), orgUnit3Node.getProperty("ametys:parentOrgUnit").getString());
        
        // orgUnit4
        Node orgUnit4Node = orgUnit4.getNode();
        Assert.assertTrue(orgUnit4Node.hasProperty("ametys:parentOrgUnit"));
        Assert.assertEquals(orgUnit2.getId(), orgUnit4Node.getProperty("ametys:parentOrgUnit").getString());
        
        // orgUnit5
        Node orgUnit5Node = orgUnit5.getNode();
        Assert.assertTrue(orgUnit5Node.hasProperty("ametys:parentOrgUnit"));
        Assert.assertEquals(orgUnit1.getId(), orgUnit5Node.getProperty("ametys:parentOrgUnit").getString());

        // program1
        Node program1Node = program1.getNode();
        Assert.assertTrue(program1Node.hasProperty("ametys:childProgramParts"));
        Assert.assertEquals(2, program1Node.getProperty("ametys:childProgramParts").getValues().length);
        Assert.assertEquals(subProgram1.getId(), program1Node.getProperty("ametys:childProgramParts").getValues()[0].getString());
        Assert.assertEquals(subProgram2.getId(), program1Node.getProperty("ametys:childProgramParts").getValues()[1].getString());
        
        // program2
        Node program2Node = program2.getNode();
        Assert.assertTrue(program2Node.hasProperty("ametys:childProgramParts"));
        Assert.assertEquals(2, program2Node.getProperty("ametys:childProgramParts").getValues().length);
        Assert.assertEquals(container1.getId(), program2Node.getProperty("ametys:childProgramParts").getValues()[0].getString());
        Assert.assertEquals(container2.getId(), program2Node.getProperty("ametys:childProgramParts").getValues()[1].getString());
        
        // program3
        Node program3Node = program3.getNode();
        Assert.assertTrue(program3Node.hasProperty("ametys:childProgramParts"));
        Assert.assertEquals(4, program3Node.getProperty("ametys:childProgramParts").getValues().length);
        Assert.assertEquals(subProgram1.getId(), program3Node.getProperty("ametys:childProgramParts").getValues()[0].getString());
        Assert.assertEquals(container1.getId(), program3Node.getProperty("ametys:childProgramParts").getValues()[1].getString());
        Assert.assertEquals(courseList1.getId(), program3Node.getProperty("ametys:childProgramParts").getValues()[2].getString());
        Assert.assertEquals(subProgram3.getId(), program3Node.getProperty("ametys:childProgramParts").getValues()[3].getString());
        
        // subProgram1
        Node subProgram1Node = subProgram1.getNode();
        Assert.assertTrue(subProgram1Node.hasProperty("ametys:childProgramParts"));
        Assert.assertEquals(3, subProgram1Node.getProperty("ametys:childProgramParts").getValues().length);
        Assert.assertEquals(container3.getId(), subProgram1Node.getProperty("ametys:childProgramParts").getValues()[0].getString());
        Assert.assertEquals(container4.getId(), subProgram1Node.getProperty("ametys:childProgramParts").getValues()[1].getString());
        Assert.assertEquals(container1.getId(), subProgram1Node.getProperty("ametys:childProgramParts").getValues()[2].getString());
        Assert.assertTrue(subProgram1Node.hasProperty("ametys:parentProgramParts"));
        Assert.assertEquals(2, subProgram1Node.getProperty("ametys:parentProgramParts").getValues().length);
        Assert.assertEquals(program1.getId(), subProgram1Node.getProperty("ametys:parentProgramParts").getValues()[0].getString());
        Assert.assertEquals(program3.getId(), subProgram1Node.getProperty("ametys:parentProgramParts").getValues()[1].getString());
        
        // subProgram2
        Node subProgram2Node = subProgram2.getNode();
        Assert.assertTrue(subProgram2Node.hasProperty("ametys:childProgramParts"));
        Assert.assertEquals(2, subProgram2Node.getProperty("ametys:childProgramParts").getValues().length);
        Assert.assertEquals(container3.getId(), subProgram2Node.getProperty("ametys:childProgramParts").getValues()[0].getString());
        Assert.assertEquals(subProgram3.getId(), subProgram2Node.getProperty("ametys:childProgramParts").getValues()[1].getString());
        Assert.assertTrue(subProgram2Node.hasProperty("ametys:parentProgramParts"));
        Assert.assertEquals(1, subProgram2Node.getProperty("ametys:parentProgramParts").getValues().length);
        Assert.assertEquals(program1.getId(), subProgram2Node.getProperty("ametys:parentProgramParts").getValues()[0].getString());
        
        // subProgram3
        Node subProgram3Node = subProgram3.getNode();
        Assert.assertTrue(subProgram3Node.hasProperty("ametys:childProgramParts"));
        Assert.assertEquals(3, subProgram3Node.getProperty("ametys:childProgramParts").getValues().length);
        Assert.assertEquals(courseList2.getId(), subProgram3Node.getProperty("ametys:childProgramParts").getValues()[0].getString());
        Assert.assertEquals(courseList3.getId(), subProgram3Node.getProperty("ametys:childProgramParts").getValues()[1].getString());
        Assert.assertEquals(courseList4.getId(), subProgram3Node.getProperty("ametys:childProgramParts").getValues()[2].getString());
        Assert.assertTrue(subProgram3Node.hasProperty("ametys:parentProgramParts"));
        Assert.assertEquals(3, subProgram3Node.getProperty("ametys:parentProgramParts").getValues().length);
        Assert.assertEquals(program3.getId(), subProgram3Node.getProperty("ametys:parentProgramParts").getValues()[0].getString());
        Assert.assertEquals(subProgram2.getId(), subProgram3Node.getProperty("ametys:parentProgramParts").getValues()[1].getString());
        Assert.assertEquals(container2.getId(), subProgram3Node.getProperty("ametys:parentProgramParts").getValues()[2].getString());
        
        // container1
        Node container1Node = container1.getNode();
        Assert.assertTrue(container1Node.hasProperty("ametys:childProgramParts"));
        Assert.assertEquals(2, container1Node.getProperty("ametys:childProgramParts").getValues().length);
        Assert.assertEquals(courseList2.getId(), container1Node.getProperty("ametys:childProgramParts").getValues()[0].getString());
        Assert.assertEquals(container5.getId(), container1Node.getProperty("ametys:childProgramParts").getValues()[1].getString());
        Assert.assertTrue(container1Node.hasProperty("ametys:parentProgramParts"));
        Assert.assertEquals(3, container1Node.getProperty("ametys:parentProgramParts").getValues().length);
        Assert.assertEquals(program2.getId(), container1Node.getProperty("ametys:parentProgramParts").getValues()[0].getString());
        Assert.assertEquals(program3.getId(), container1Node.getProperty("ametys:parentProgramParts").getValues()[1].getString());
        Assert.assertEquals(subProgram1.getId(), container1Node.getProperty("ametys:parentProgramParts").getValues()[2].getString());
        
        // container2
        Node container2Node = container2.getNode();
        Assert.assertTrue(container2Node.hasProperty("ametys:childProgramParts"));
        Assert.assertEquals(1, container2Node.getProperty("ametys:childProgramParts").getValues().length);
        Assert.assertEquals(subProgram3.getId(), container2Node.getProperty("ametys:childProgramParts").getValues()[0].getString());
        Assert.assertTrue(container2Node.hasProperty("ametys:parentProgramParts"));
        Assert.assertEquals(1, container2Node.getProperty("ametys:parentProgramParts").getValues().length);
        Assert.assertEquals(program2.getId(), container2Node.getProperty("ametys:parentProgramParts").getValues()[0].getString());
        
        // container3
        Node container3Node = container3.getNode();
        Assert.assertTrue(container3Node.hasProperty("ametys:parentProgramParts"));
        Assert.assertEquals(2, container3Node.getProperty("ametys:parentProgramParts").getValues().length);
        Assert.assertEquals(subProgram1.getId(), container3Node.getProperty("ametys:parentProgramParts").getValues()[0].getString());
        Assert.assertEquals(subProgram2.getId(), container3Node.getProperty("ametys:parentProgramParts").getValues()[1].getString());
        
        // container4
        Node container4Node = container4.getNode();
        Assert.assertTrue(container4Node.hasProperty("ametys:parentProgramParts"));
        Assert.assertEquals(1, container4Node.getProperty("ametys:parentProgramParts").getValues().length);
        Assert.assertEquals(subProgram1.getId(), container4Node.getProperty("ametys:parentProgramParts").getValues()[0].getString());
        
        // container5
        Node container5Node = container5.getNode();
        Assert.assertTrue(container5Node.hasProperty("ametys:parentProgramParts"));
        Assert.assertEquals(1, container5Node.getProperty("ametys:parentProgramParts").getValues().length);
        Assert.assertEquals(container1.getId(), container5Node.getProperty("ametys:parentProgramParts").getValues()[0].getString());
        
        // courseList1
        Node courseList1Node = courseList1.getNode();
        Assert.assertTrue(courseList1Node.hasProperty("ametys:courses"));
        Assert.assertEquals(2, courseList1Node.getProperty("ametys:courses").getValues().length);
        Assert.assertEquals(course1.getId(), courseList1Node.getProperty("ametys:courses").getValues()[0].getString());
        Assert.assertEquals(course2.getId(), courseList1Node.getProperty("ametys:courses").getValues()[1].getString());
        Assert.assertTrue(courseList1Node.hasProperty("ametys:parentProgramParts"));
        Assert.assertEquals(1, courseList1Node.getProperty("ametys:parentProgramParts").getValues().length);
        Assert.assertEquals(program3.getId(), courseList1Node.getProperty("ametys:parentProgramParts").getValues()[0].getString());
        
        // courseList2
        Node courseList2Node = courseList2.getNode();
        Assert.assertTrue(courseList2Node.hasProperty("ametys:courses"));
        Assert.assertEquals(3, courseList2Node.getProperty("ametys:courses").getValues().length);
        Assert.assertEquals(course1.getId(), courseList2Node.getProperty("ametys:courses").getValues()[0].getString());
        Assert.assertEquals(course3.getId(), courseList2Node.getProperty("ametys:courses").getValues()[1].getString());
        Assert.assertEquals(course4.getId(), courseList2Node.getProperty("ametys:courses").getValues()[2].getString());
        Assert.assertTrue(courseList2Node.hasProperty("ametys:parentProgramParts"));
        Assert.assertEquals(2, courseList2Node.getProperty("ametys:parentProgramParts").getValues().length);
        Assert.assertEquals(subProgram3.getId(), courseList2Node.getProperty("ametys:parentProgramParts").getValues()[0].getString());
        Assert.assertEquals(container1.getId(), courseList2Node.getProperty("ametys:parentProgramParts").getValues()[1].getString());
        
        // courseList3
        Node courseList3Node = courseList3.getNode();
        Assert.assertTrue(courseList3Node.hasProperty("ametys:courses"));
        Assert.assertEquals(2, courseList3Node.getProperty("ametys:courses").getValues().length);
        Assert.assertEquals(course5.getId(), courseList3Node.getProperty("ametys:courses").getValues()[0].getString());
        Assert.assertEquals(course6.getId(), courseList3Node.getProperty("ametys:courses").getValues()[1].getString());
        Assert.assertTrue(courseList3Node.hasProperty("ametys:parentCourses"));
        Assert.assertEquals(2, courseList3Node.getProperty("ametys:parentCourses").getValues().length);
        Assert.assertEquals(course1.getId(), courseList3Node.getProperty("ametys:parentCourses").getValues()[0].getString());
        Assert.assertEquals(course2.getId(), courseList3Node.getProperty("ametys:parentCourses").getValues()[1].getString());
        Assert.assertTrue(courseList3Node.hasProperty("ametys:parentProgramParts"));
        Assert.assertEquals(1, courseList3Node.getProperty("ametys:parentProgramParts").getValues().length);
        Assert.assertEquals(subProgram3.getId(), courseList3Node.getProperty("ametys:parentProgramParts").getValues()[0].getString());
        
        // courseList4
        Node courseList4Node = courseList4.getNode();
        Assert.assertTrue(courseList4Node.hasProperty("ametys:courses"));
        Assert.assertEquals(2, courseList4Node.getProperty("ametys:courses").getValues().length);
        Assert.assertEquals(course3.getId(), courseList4Node.getProperty("ametys:courses").getValues()[0].getString());
        Assert.assertEquals(course7.getId(), courseList4Node.getProperty("ametys:courses").getValues()[1].getString());
        Assert.assertTrue(courseList4Node.hasProperty("ametys:parentCourses"));
        Assert.assertEquals(1, courseList4Node.getProperty("ametys:parentCourses").getValues().length);
        Assert.assertEquals(course2.getId(), courseList4Node.getProperty("ametys:parentCourses").getValues()[0].getString());
        Assert.assertTrue(courseList4Node.hasProperty("ametys:parentProgramParts"));
        Assert.assertEquals(1, courseList4Node.getProperty("ametys:parentProgramParts").getValues().length);
        Assert.assertEquals(subProgram3.getId(), courseList4Node.getProperty("ametys:parentProgramParts").getValues()[0].getString());

        // course1
        Node course1Node = course1.getNode();
        Assert.assertTrue(course1Node.hasProperty("ametys:parentCourseLists"));
        Assert.assertEquals(2, course1Node.getProperty("ametys:parentCourseLists").getValues().length);
        Assert.assertEquals(courseList1.getId(), course1Node.getProperty("ametys:parentCourseLists").getValues()[0].getString());
        Assert.assertEquals(courseList2.getId(), course1Node.getProperty("ametys:parentCourseLists").getValues()[1].getString());
        Assert.assertTrue(course1Node.hasProperty("ametys:courseLists"));
        Assert.assertEquals(1, course1Node.getProperty("ametys:courseLists").getValues().length);
        Assert.assertEquals(courseList3.getId(), course1Node.getProperty("ametys:courseLists").getValues()[0].getString());
        
        // course2
        Node course2Node = course2.getNode();
        Assert.assertTrue(course2Node.hasProperty("ametys:parentCourseLists"));
        Assert.assertEquals(1, course2Node.getProperty("ametys:parentCourseLists").getValues().length);
        Assert.assertEquals(courseList1.getId(), course2Node.getProperty("ametys:parentCourseLists").getValues()[0].getString());
        Assert.assertTrue(course2Node.hasProperty("ametys:courseLists"));
        Assert.assertEquals(2, course2Node.getProperty("ametys:courseLists").getValues().length);
        Assert.assertEquals(courseList3.getId(), course2Node.getProperty("ametys:courseLists").getValues()[0].getString());
        Assert.assertEquals(courseList4.getId(), course2Node.getProperty("ametys:courseLists").getValues()[1].getString());
        
        // course3
        Node course3Node = course3.getNode();
        Assert.assertTrue(course3Node.hasProperty("ametys:parentCourseLists"));
        Assert.assertEquals(2, course3Node.getProperty("ametys:parentCourseLists").getValues().length);
        Assert.assertEquals(courseList2.getId(), course3Node.getProperty("ametys:parentCourseLists").getValues()[0].getString());
        Assert.assertEquals(courseList4.getId(), course3Node.getProperty("ametys:parentCourseLists").getValues()[1].getString());
        
        // course4
        Node course4Node = course4.getNode();
        Assert.assertTrue(course4Node.hasProperty("ametys:parentCourseLists"));
        Assert.assertEquals(1, course4Node.getProperty("ametys:parentCourseLists").getValues().length);
        Assert.assertEquals(courseList2.getId(), course4Node.getProperty("ametys:parentCourseLists").getValues()[0].getString());
        
        // course5
        Node course5Node = course5.getNode();
        Assert.assertTrue(course5Node.hasProperty("ametys:parentCourseLists"));
        Assert.assertEquals(1, course5Node.getProperty("ametys:parentCourseLists").getValues().length);
        Assert.assertEquals(courseList3.getId(), course5Node.getProperty("ametys:parentCourseLists").getValues()[0].getString());
        
        // course6
        Node course6Node = course6.getNode();
        Assert.assertTrue(course6Node.hasProperty("ametys:parentCourseLists"));
        Assert.assertEquals(1, course6Node.getProperty("ametys:parentCourseLists").getValues().length);
        Assert.assertEquals(courseList3.getId(), course6Node.getProperty("ametys:parentCourseLists").getValues()[0].getString());
        
        // course7
        Node course7Node = course7.getNode();
        Assert.assertTrue(course7Node.hasProperty("ametys:parentCourseLists"));
        Assert.assertEquals(1, course7Node.getProperty("ametys:parentCourseLists").getValues().length);
        Assert.assertEquals(courseList4.getId(), course7Node.getProperty("ametys:parentCourseLists").getValues()[0].getString());
    }
}
