/*
 *  Copyright 2016 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.plugins.odfweb;

import java.util.List;
import java.util.Map;

import javax.jcr.Value;

import org.apache.jackrabbit.value.StringValue;
import org.junit.Assert;
import org.junit.Test;

import org.ametys.cms.repository.Content;
import org.ametys.cms.repository.WorkflowAwareContent;
import org.ametys.odf.ProgramItem;
import org.ametys.odf.course.Course;
import org.ametys.odf.courselist.CourseList;
import org.ametys.odf.courselist.CourseList.ChoiceType;
import org.ametys.odf.enumeration.OdfReferenceTableHelper;
import org.ametys.odf.orgunit.OrgUnit;
import org.ametys.odf.program.AbstractProgram;
import org.ametys.odf.program.Container;
import org.ametys.odf.program.Program;
import org.ametys.odf.program.SubProgram;
import org.ametys.plugins.odfweb.repository.FirstLevelPageFactory;
import org.ametys.plugins.odfweb.repository.OdfPageHandler;
import org.ametys.plugins.repository.AmetysObjectResolver;
import org.ametys.plugins.repository.UnknownAmetysObjectException;
import org.ametys.runtime.test.framework.CocoonTestCase.NeedFeatures;
import org.ametys.web.repository.page.Page;
import org.ametys.web.repository.page.jcr.DefaultPage;
import org.ametys.web.repository.page.jcr.DefaultPageFactory;
import org.ametys.web.repository.sitemap.Sitemap;

/**
 * Test case for virtual ODF pages
 */
@NeedFeatures({
    // Direct dependencies (components)
    "repository/provider.jackrabbit",
    "core/model.helper",
    "repository/repository.data.holder.helpers",
    "workflow/condition.current.step",
    
    // odfweb
    "odf-web/odf.virtualfactory",
    "odf-web/odf.web.site.parameters",
})
public class ODFVirtualPageTestCase extends AbstractOdfWebTestCase
{
    private static final String __CATALOG_NAME = "2017";
    
    /**
     * Test if the ODF virtual pages are correctly created with the right structure.
     * @throws Exception if an error occurs.
     */
    @Test
    public void testVirtualPages() throws Exception
    {
        _configureSitemaps("test/environments/sitemap.xml");
        
        // Create Catalog
        _catalogDAO.createCatalog("Catalogue 2017", __CATALOG_NAME, null);
        
        // Create OrgUnit
        OrgUnit orgUnit1 = _odfContentHelper.new OrgUnitBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "Composante TU 1", OrgUnit.CODE_UAI, "UAITU1")).build();
        
        // Create Program
        String domainId = _odfReferenceTableHelper.getItemFromCode(OdfReferenceTableHelper.DOMAIN, __DOMAIN_ALL).getId();
        Program program1 = _odfContentHelper.new ProgramBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "Formation TU 1", ProgramItem.ORG_UNITS_REFERENCES, List.of(orgUnit1), "domain2", domainId)).withDefaultCatalog(__CATALOG_NAME).build();
        Program program2 = _odfContentHelper.new ProgramBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "Formation TU 2", ProgramItem.ORG_UNITS_REFERENCES, List.of(orgUnit1), "domain2", domainId)).withDefaultCatalog(__CATALOG_NAME).build();
        Program program3 = _odfContentHelper.new ProgramBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "Formation TU 3", ProgramItem.ORG_UNITS_REFERENCES, List.of(orgUnit1), AbstractProgram.DEGREE, __DEGREE_B, "domain2", domainId)).withDefaultCatalog(__CATALOG_NAME).build();
        
        // Create SubProgram
        SubProgram subProgram1 = _odfContentHelper.new SubProgramBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "Parcours TU 1", ProgramItem.ORG_UNITS_REFERENCES, List.of(orgUnit1))).withDefaultCatalog(__CATALOG_NAME).build();
        SubProgram subProgram2 = _odfContentHelper.new SubProgramBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "Parcours TU 2", ProgramItem.ORG_UNITS_REFERENCES, List.of(orgUnit1))).withDefaultCatalog(__CATALOG_NAME).build();
        SubProgram subProgram3 = _odfContentHelper.new SubProgramBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "Parcours TU 3", ProgramItem.ORG_UNITS_REFERENCES, List.of(orgUnit1))).withDefaultCatalog(__CATALOG_NAME).build();

        // Create Container
        Container container1 = _odfContentHelper.new ContainerBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "Conteneur TU 1")).withDefaultCatalog(__CATALOG_NAME).build();
        Container container2 = _odfContentHelper.new ContainerBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "Conteneur TU 2", Container.PERIOD, __PERIOD_SEM_1, Container.NATURE, __CONTAINER_NATURE_SEM)).withDefaultCatalog(__CATALOG_NAME).build();
        Container container3 = _odfContentHelper.new ContainerBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "Conteneur TU 3", Container.PERIOD, __PERIOD_SEM_1, Container.NATURE, __CONTAINER_NATURE_SEM)).withDefaultCatalog(__CATALOG_NAME).build();
        Container container4 = _odfContentHelper.new ContainerBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "Conteneur TU 4", Container.PERIOD, __PERIOD_SEM_2, Container.NATURE, __CONTAINER_NATURE_SEM)).withDefaultCatalog(__CATALOG_NAME).build();
        Container container5 = _odfContentHelper.new ContainerBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "Conteneur TU 5", Container.PERIOD, __PERIOD_SEM_2, Container.NATURE, __CONTAINER_NATURE_SEM)).withDefaultCatalog(__CATALOG_NAME).build();
        
        // Create CourseList
        CourseList courseList1 = _odfContentHelper.new CourseListBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "Liste TU 1", CourseList.CHOICE_TYPE, ChoiceType.OPTIONAL)).withDefaultCatalog(__CATALOG_NAME).build();
        CourseList courseList2 = _odfContentHelper.new CourseListBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "Liste TU 2")).withDefaultCatalog(__CATALOG_NAME).build();
        CourseList courseList3 = _odfContentHelper.new CourseListBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "Liste TU 3")).withDefaultCatalog(__CATALOG_NAME).build();
        CourseList courseList4 = _odfContentHelper.new CourseListBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "Liste TU 4")).withDefaultCatalog(__CATALOG_NAME).build();

        // Create Course
        Course course1 = _odfContentHelper.new CourseBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "ELP TU 1", Course.COURSE_TYPE, __COURSE_NATURE_EC)).withDefaultCatalog(__CATALOG_NAME).build();
        Course course2 = _odfContentHelper.new CourseBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "ELP TU 2", Course.COURSE_TYPE, __COURSE_NATURE_EC)).withDefaultCatalog(__CATALOG_NAME).build();
        Course course3 = _odfContentHelper.new CourseBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "ELP TU 3", Course.COURSE_TYPE, __COURSE_NATURE_EC)).withDefaultCatalog(__CATALOG_NAME).build();
        Course course4 = _odfContentHelper.new CourseBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "ELP TU 4", Course.COURSE_TYPE, __COURSE_NATURE_EC)).withDefaultCatalog(__CATALOG_NAME).build();
        Course course5 = _odfContentHelper.new CourseBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "ELP TU 5")).withDefaultCatalog(__CATALOG_NAME).build();
        Course course6 = _odfContentHelper.new CourseBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "ELP TU 6")).withDefaultCatalog(__CATALOG_NAME).build();
        Course course7 = _odfContentHelper.new CourseBuilder().fromJavaMap(Map.of(Content.ATTRIBUTE_TITLE, "ELP TU 7")).withDefaultCatalog(__CATALOG_NAME).build();

        // Create structure
        _odfContentHelper.editContent(courseList1, Map.of("courses", List.of(course1.getId(), course2.getId())));
        _odfContentHelper.editContent(courseList2, Map.of("courses", List.of(course1.getId(), course3.getId(), course4.getId())));
        _odfContentHelper.editContent(courseList3, Map.of("courses", List.of(course5.getId(), course6.getId())));
        _odfContentHelper.editContent(courseList4, Map.of("courses", List.of(course3.getId(), course7.getId())));
        _odfContentHelper.editContent(course1, Map.of("courseLists", List.of(courseList3.getId())));
        _odfContentHelper.editContent(course2, Map.of("courseLists", List.of(courseList3.getId(), courseList4.getId())));
        _odfContentHelper.editContent(program1, Map.of("childProgramParts", List.of(subProgram1.getId(), subProgram2.getId())));
        _odfContentHelper.editContent(program2, Map.of("childProgramParts", List.of(container1.getId(), container2.getId())));
        _odfContentHelper.editContent(program3, Map.of("childProgramParts", List.of(subProgram1.getId(), container1.getId(), courseList1.getId(), subProgram3.getId())));
        _odfContentHelper.editContent(subProgram1, Map.of("childProgramParts", List.of(container3.getId(), container4.getId(), container1.getId())));
        _odfContentHelper.editContent(subProgram2, Map.of("childProgramParts", List.of(container3.getId(), subProgram3.getId())));
        _odfContentHelper.editContent(subProgram3, Map.of("childProgramParts", List.of(courseList2.getId(), courseList3.getId(), courseList4.getId())));
        _odfContentHelper.editContent(container1, Map.of("childProgramParts", List.of(courseList2.getId(), container5.getId())));
        _odfContentHelper.editContent(container2, Map.of("childProgramParts", List.of(subProgram3.getId())));
        
        // Validate contents
        _odfContentHelper.validateContents(orgUnit1,
                program1, program2, program3,
                subProgram1, subProgram2, subProgram3,
                container1, container2, container3, container4, container5,
                courseList1, courseList2, courseList3, courseList4,
                course1, course2, course3, course4, course5, course6, course7);
        
        // Set ODF root page
        Sitemap sitemap = _siteManager.getSite("www").getSitemap("fr");
        DefaultPage<DefaultPageFactory> page = sitemap.getChild("section1");
        page.setValue(OdfPageHandler.CATALOG_DATA_NAME, __CATALOG_NAME);
        page.setValue(OdfPageHandler.LEVEL1_ATTRIBUTE_NAME, "domain2");
        page.setValue(OdfPageHandler.LEVEL2_ATTRIBUTE_NAME, "degree");
        
        Value[] value = new Value[] {new StringValue(FirstLevelPageFactory.class.getName())};
        page.getNode().setProperty(AmetysObjectResolver.VIRTUAL_PROPERTY, value);
        
        _testVirtualPagesWith2Levels(program1, program2, program3, subProgram1, subProgram2, subProgram3, course1, course2, course3, course4, course5, course6, course7, page);

        page.removeValue(OdfPageHandler.LEVEL2_ATTRIBUTE_NAME);
        _testVirtualPagesWith1Level(program1, program2, program3, subProgram1, subProgram2, subProgram3, course1, course2, course3, course4, course5, course6, course7, page);
        
        page.removeValue(OdfPageHandler.LEVEL1_ATTRIBUTE_NAME);
        _testVirtualPagesWithNoLevel(program1, program2, program3, subProgram1, subProgram2, subProgram3, course1, course2, course3, course4, course5, course6, course7, page);

    }
    
    private void _testVirtualPagesWith2Levels(WorkflowAwareContent program1, WorkflowAwareContent program2, WorkflowAwareContent program3, WorkflowAwareContent subProgram1,
            WorkflowAwareContent subProgram2, WorkflowAwareContent subProgram3, WorkflowAwareContent course1, WorkflowAwareContent course2, WorkflowAwareContent course3,
            WorkflowAwareContent course4, WorkflowAwareContent course5, WorkflowAwareContent course6, WorkflowAwareContent course7, DefaultPage<DefaultPageFactory> page)
    {
        String levelPath = "section1/arts-lettres-langues-ALL/licence-lmd-XA/";
        
        _assertProgram1Pages(program1, subProgram1, subProgram2, subProgram3, course1, course3, course4, course5, course6, course7, page, levelPath);
        _assertProgram2Pages(program2, subProgram3, course1, course3, course4, course5, course6, course7, page, levelPath);
        
        levelPath = "section1/arts-lettres-langues-ALL/master-lmd-XB/";
        _assertProgram3Pages(program3, subProgram1, subProgram3, course1, course2, course3, course4, course5, course6, course7, page, levelPath);
    }
    
    private void _testVirtualPagesWith1Level(WorkflowAwareContent program1, WorkflowAwareContent program2, WorkflowAwareContent program3, WorkflowAwareContent subProgram1,
            WorkflowAwareContent subProgram2, WorkflowAwareContent subProgram3, WorkflowAwareContent course1, WorkflowAwareContent course2, WorkflowAwareContent course3,
            WorkflowAwareContent course4, WorkflowAwareContent course5, WorkflowAwareContent course6, WorkflowAwareContent course7, DefaultPage<DefaultPageFactory> page)
    {
        String levelPath = "section1/arts-lettres-langues-ALL/";
        
        _assertProgram1Pages(program1, subProgram1, subProgram2, subProgram3, course1, course3, course4, course5, course6, course7, page, levelPath);
        _assertProgram2Pages(program2, subProgram3, course1, course3, course4, course5, course6, course7, page, levelPath);
        _assertProgram3Pages(program3, subProgram1, subProgram3, course1, course2, course3, course4, course5, course6, course7, page, levelPath);
    }
    
    private void _testVirtualPagesWithNoLevel(WorkflowAwareContent program1, WorkflowAwareContent program2, WorkflowAwareContent program3, WorkflowAwareContent subProgram1,
            WorkflowAwareContent subProgram2, WorkflowAwareContent subProgram3, WorkflowAwareContent course1, WorkflowAwareContent course2, WorkflowAwareContent course3,
            WorkflowAwareContent course4, WorkflowAwareContent course5, WorkflowAwareContent course6, WorkflowAwareContent course7, DefaultPage<DefaultPageFactory> page)
    {
        String levelPath = "section1/";
        
        _assertProgram1Pages(program1, subProgram1, subProgram2, subProgram3, course1, course3, course4, course5, course6, course7, page, levelPath);
        _assertProgram2Pages(program2, subProgram3, course1, course3, course4, course5, course6, course7, page, levelPath);
        _assertProgram3Pages(program3, subProgram1, subProgram3, course1, course2, course3, course4, course5, course6, course7, page, levelPath);
    }
    
    private void _assertProgram1Pages(WorkflowAwareContent program1, WorkflowAwareContent subProgram1,
            WorkflowAwareContent subProgram2, WorkflowAwareContent subProgram3, WorkflowAwareContent course1, WorkflowAwareContent course3,
            WorkflowAwareContent course4, WorkflowAwareContent course5, WorkflowAwareContent course6, WorkflowAwareContent course7, DefaultPage<DefaultPageFactory> page, String levelPath)
    {
        // Contrôle du program1
        /* Arborescence de la formation
        program1
            subprogram1
                container3
                container4
                container1
                    courselist2
                        course1
                            courselist3
                                course5
                                course6
                        course3
                        course4
                    container5
            subprogram2
                container3
                subprogram3
                    courselist2
                        course1
                            courselist3
                                course5
                                course6
                        course3
                        course4
                    courselist3
                        course5
                        course6
                    courselist4
                        course3
                        course7
        */
        /* Arborescence de pages
        program1
            subprogram1
                course1
                    course5
                    course6
                course3
                course4
            subprogram2
                subprogram3
                    course1
                        course5
                        course6
                    course3
                    course4
                    course5
                    course6
                    course7
         */
        
        Page programPage = null;
        try
        {
            programPage = _ametysObjectResolver.resolveById("program://_root?rootId=" + page.getId() + "&programId=" + program1.getId());
        }
        catch (UnknownAmetysObjectException e)
        {
            Assert.fail("Failed to resolve virtual ODF page with id '" + "program://_root?rootId=" + page.getId() + "&programId=" + program1.getId() + "'.");
            return; // never will go there as #fail will...fail (but need it to avoid a compile-time warning on programPage possibly null)
        }
        
        String rootId = "rootId=" + page.getId();
        String parentId = "&parentId=" + program1.getId();
        String programId = "&programId=" + program1.getId();

        // Contrôle de l'arborescence des pages de program1 (voir ci-dessus)
        _assertPage(programPage, "formation-tu-1-[0-9A-Z]{8}", "program", rootId + programId, null, levelPath, 2);
        _assertPage(programPage.getChildPageAt(0), "parcours-tu-1-[0-9A-Z]{8}", "program", rootId + "&programId=" + subProgram1.getId() + parentId, "formation-tu-1-[0-9A-Z]{8}", levelPath, 3); // Page de subProgram1 avec 3 enfants et enfant de program1
        _assertPage(programPage.getChildPageAt(0).getChildPageAt(0), "elp-tu-1-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course1.getId() + programId, "formation-tu-1-[0-9A-Z]{8}/parcours-tu-1-[0-9A-Z]{8}", levelPath, 2); // Page de course1 avec 2 enfants et enfant de program1/subProgram1
        _assertPage(programPage.getChildPageAt(0).getChildPageAt(0).getChildPageAt(0), "elp-tu-5-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course5.getId() + programId, "formation-tu-1-[0-9A-Z]{8}/parcours-tu-1-[0-9A-Z]{8}/elp-tu-1-[0-9A-Z]{8}", levelPath, 0); // Page de course5 sans enfant et enfant de program1/subProgram1/course1
        _assertPage(programPage.getChildPageAt(0).getChildPageAt(0).getChildPageAt(1), "elp-tu-6-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course6.getId() + programId, "formation-tu-1-[0-9A-Z]{8}/parcours-tu-1-[0-9A-Z]{8}/elp-tu-1-[0-9A-Z]{8}", levelPath, 0); // Page de course6 sans enfant et enfant de program1/subProgram1/course1
        _assertPage(programPage.getChildPageAt(0).getChildPageAt(1), "elp-tu-3-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course3.getId() + programId, "formation-tu-1-[0-9A-Z]{8}/parcours-tu-1-[0-9A-Z]{8}", levelPath, 0); // Page de course3 sans enfant et enfant de program1/subProgram1
        _assertPage(programPage.getChildPageAt(0).getChildPageAt(2), "elp-tu-4-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course4.getId() + programId, "formation-tu-1-[0-9A-Z]{8}/parcours-tu-1-[0-9A-Z]{8}", levelPath, 0); // Page de course4 sans enfant et enfant de program1/subProgram1
        _assertPage(programPage.getChildPageAt(1), "parcours-tu-2-[0-9A-Z]{8}", "program", rootId + "&programId=" + subProgram2.getId() + parentId, "formation-tu-1-[0-9A-Z]{8}", levelPath, 1); // Page de subProgram2 avec 1 enfant et enfant de program1
        _assertPage(programPage.getChildPageAt(1).getChildPageAt(0), "parcours-tu-3-[0-9A-Z]{8}", "program", rootId + "&programId=" + subProgram3.getId() + parentId, "formation-tu-1-[0-9A-Z]{8}/parcours-tu-2-[0-9A-Z]{8}", levelPath, 6); // Page de subProgram3 avec 6 enfants et enfant de program1/subProgram2
        _assertPage(programPage.getChildPageAt(1).getChildPageAt(0).getChildPageAt(0), "elp-tu-1-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course1.getId() + programId, "formation-tu-1-[0-9A-Z]{8}/parcours-tu-2-[0-9A-Z]{8}/parcours-tu-3-[0-9A-Z]{8}", levelPath, 2); // Page de course1 avec 2 enfants et enfant de program1/subProgram2/subProgram3
        _assertPage(programPage.getChildPageAt(1).getChildPageAt(0).getChildPageAt(0).getChildPageAt(0), "elp-tu-5-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course5.getId() + programId, "formation-tu-1-[0-9A-Z]{8}/parcours-tu-2-[0-9A-Z]{8}/parcours-tu-3-[0-9A-Z]{8}/elp-tu-1-[0-9A-Z]{8}", levelPath, 0); // Page de course5 sans enfant et enfant de program1/subProgram2/subProgram3/course1
        _assertPage(programPage.getChildPageAt(1).getChildPageAt(0).getChildPageAt(0).getChildPageAt(1), "elp-tu-6-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course6.getId() + programId, "formation-tu-1-[0-9A-Z]{8}/parcours-tu-2-[0-9A-Z]{8}/parcours-tu-3-[0-9A-Z]{8}/elp-tu-1-[0-9A-Z]{8}", levelPath, 0); // Page de course6 sans enfant et enfant de program1/subProgram2/subProgram3/course1
        _assertPage(programPage.getChildPageAt(1).getChildPageAt(0).getChildPageAt(1), "elp-tu-3-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course3.getId() + programId, "formation-tu-1-[0-9A-Z]{8}/parcours-tu-2-[0-9A-Z]{8}/parcours-tu-3-[0-9A-Z]{8}", levelPath, 0); // Page de course3 sans enfant et enfant de program1/subProgram2/subProgram3
        _assertPage(programPage.getChildPageAt(1).getChildPageAt(0).getChildPageAt(2), "elp-tu-4-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course4.getId() + programId, "formation-tu-1-[0-9A-Z]{8}/parcours-tu-2-[0-9A-Z]{8}/parcours-tu-3-[0-9A-Z]{8}", levelPath, 0); // Page de course4 sans enfant et enfant de program1/subProgram2/subProgram3
        _assertPage(programPage.getChildPageAt(1).getChildPageAt(0).getChildPageAt(3), "elp-tu-5-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course5.getId() + programId, "formation-tu-1-[0-9A-Z]{8}/parcours-tu-2-[0-9A-Z]{8}/parcours-tu-3-[0-9A-Z]{8}", levelPath, 0); // Page de course5 sans enfant et enfant de program1/subProgram2/subProgram3
        _assertPage(programPage.getChildPageAt(1).getChildPageAt(0).getChildPageAt(4), "elp-tu-6-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course6.getId() + programId, "formation-tu-1-[0-9A-Z]{8}/parcours-tu-2-[0-9A-Z]{8}/parcours-tu-3-[0-9A-Z]{8}", levelPath, 0); // Page de course6 sans enfant et enfant de program1/subProgram2/subProgram3
        _assertPage(programPage.getChildPageAt(1).getChildPageAt(0).getChildPageAt(5), "elp-tu-7-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course7.getId() + programId, "formation-tu-1-[0-9A-Z]{8}/parcours-tu-2-[0-9A-Z]{8}/parcours-tu-3-[0-9A-Z]{8}", levelPath, 0); // Page de course7 sans enfant et enfant de program1/subProgram2/subProgram3
    }
    
    private void _assertProgram2Pages(WorkflowAwareContent program2, WorkflowAwareContent subProgram3, WorkflowAwareContent course1, WorkflowAwareContent course3,
            WorkflowAwareContent course4, WorkflowAwareContent course5, WorkflowAwareContent course6, WorkflowAwareContent course7, DefaultPage<DefaultPageFactory> page, String levelPath)
    {
        // Contrôle du program2
        /* Arborescence de la formation
        program2
            container1
                courselist2
                    course1
                        courselist3
                            course5
                            course6
                    course3
                    course4
                container5
            container2
                subprogram3
                    courselist2
                        course1
                            courselist3
                                course5
                                course6
                        course3
                        course4
                    courselist3
                        course5
                        course6
                    courselist4
                        course3
                        course7
        */
        /* Arborescence de pages
        program2
            course1
                course5
                course6
            course3
            course4
            subprogram3
                course1
                    course5
                    course6
                course3
                course4
                course5
                course6
                course7
        */
        
        Page programPage = null;
        try
        {
            programPage = _ametysObjectResolver.resolveById("program://_root?rootId=" + page.getId() + "&programId=" + program2.getId());
        }
        catch (UnknownAmetysObjectException e)
        {
            Assert.fail("Failed to resolve virtual ODF page with id '" + "program://_root?rootId=" + page.getId() + "&programId=" + program2.getId() + "'.");
        }
        
        String rootId = "rootId=" + page.getId();
        String parentId = "&parentId=" + program2.getId();
        String programId = "&programId=" + program2.getId();
        
        // Contrôle de l'arborescence des pages de program2 (voir ci-dessus)
        Assert.assertNotNull(programPage);
        _assertPage(programPage, "formation-tu-2-[0-9A-Z]{8}", "program", rootId + programId, null, levelPath, 4); // Page de program2 avec 4 enfants et pas de parent
        _assertPage(programPage.getChildPageAt(0), "elp-tu-1-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course1.getId() + programId, "formation-tu-2-[0-9A-Z]{8}", levelPath, 2); // Page de course1 avec 2 enfants et enfant de program2
        _assertPage(programPage.getChildPageAt(0).getChildPageAt(0), "elp-tu-5-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course5.getId() + programId, "formation-tu-2-[0-9A-Z]{8}/elp-tu-1-[0-9A-Z]{8}", levelPath, 0); // Page de course5 sans enfant et enfant de program2/course1
        _assertPage(programPage.getChildPageAt(0).getChildPageAt(1), "elp-tu-6-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course6.getId() + programId, "formation-tu-2-[0-9A-Z]{8}/elp-tu-1-[0-9A-Z]{8}", levelPath, 0); // Page de course6 sans enfant et enfant de program2/course1
        _assertPage(programPage.getChildPageAt(1), "elp-tu-3-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course3.getId() + programId, "formation-tu-2-[0-9A-Z]{8}", levelPath, 0); // Page de course3 sans enfant et enfant de program2
        _assertPage(programPage.getChildPageAt(2), "elp-tu-4-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course4.getId() + programId, "formation-tu-2-[0-9A-Z]{8}", levelPath, 0); // Page de course4 sans enfant et enfant de program2
        _assertPage(programPage.getChildPageAt(3), "parcours-tu-3-[0-9A-Z]{8}", "program", rootId + "&programId=" + subProgram3.getId() + parentId, "formation-tu-2-[0-9A-Z]{8}", levelPath, 6); // Page de subProgram3 avec 6 enfants et enfant de program2
        _assertPage(programPage.getChildPageAt(3).getChildPageAt(0), "elp-tu-1-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course1.getId() + programId, "formation-tu-2-[0-9A-Z]{8}/parcours-tu-3-[0-9A-Z]{8}", levelPath, 2); // Page de course1 avec 2 enfants et enfant de program2/subProgram3
        _assertPage(programPage.getChildPageAt(3).getChildPageAt(0).getChildPageAt(0), "elp-tu-5-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course5.getId() + programId, "formation-tu-2-[0-9A-Z]{8}/parcours-tu-3-[0-9A-Z]{8}/elp-tu-1-[0-9A-Z]{8}", levelPath, 0); // Page de course5 sans enfant et enfant de program2/subProgram3/course1
        _assertPage(programPage.getChildPageAt(3).getChildPageAt(0).getChildPageAt(1), "elp-tu-6-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course6.getId() + programId, "formation-tu-2-[0-9A-Z]{8}/parcours-tu-3-[0-9A-Z]{8}/elp-tu-1-[0-9A-Z]{8}", levelPath, 0); // Page de course5 sans enfant et enfant de program2/subProgram3/course1
        _assertPage(programPage.getChildPageAt(3).getChildPageAt(1), "elp-tu-3-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course3.getId() + programId, "formation-tu-2-[0-9A-Z]{8}/parcours-tu-3-[0-9A-Z]{8}", levelPath, 0); // Page de course3 sans enfant et enfant de program2/subProgram3
        _assertPage(programPage.getChildPageAt(3).getChildPageAt(2), "elp-tu-4-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course4.getId() + programId, "formation-tu-2-[0-9A-Z]{8}/parcours-tu-3-[0-9A-Z]{8}", levelPath, 0); // Page de course4 sans enfant et enfant de program2/subProgram3
        _assertPage(programPage.getChildPageAt(3).getChildPageAt(3), "elp-tu-5-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course5.getId() + programId, "formation-tu-2-[0-9A-Z]{8}/parcours-tu-3-[0-9A-Z]{8}", levelPath, 0); // Page de course5 sans enfant et enfant de program2/subProgram3
        _assertPage(programPage.getChildPageAt(3).getChildPageAt(4), "elp-tu-6-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course6.getId() + programId, "formation-tu-2-[0-9A-Z]{8}/parcours-tu-3-[0-9A-Z]{8}", levelPath, 0); // Page de course6 sans enfant et enfant de program2/subProgram3
        _assertPage(programPage.getChildPageAt(3).getChildPageAt(5), "elp-tu-7-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course7.getId() + programId, "formation-tu-2-[0-9A-Z]{8}/parcours-tu-3-[0-9A-Z]{8}", levelPath, 0); // Page de course7 sans enfant et enfant de program2/subProgram3
    }
    
    private void _assertProgram3Pages(WorkflowAwareContent program3, WorkflowAwareContent subProgram1,
            WorkflowAwareContent subProgram3, WorkflowAwareContent course1, WorkflowAwareContent course2, WorkflowAwareContent course3,
            WorkflowAwareContent course4, WorkflowAwareContent course5, WorkflowAwareContent course6, WorkflowAwareContent course7, DefaultPage<DefaultPageFactory> page, String levelPath)
    {
        // Contrôle du program3
        /* Arborescence de la formation
        program3
            subprogram1
                container3
                container4
                container1
                    courselist2
                        course1
                            courselist3
                                course5
                                course6
                        course3
                        course4
                    container5
            container1
                courselist2
                    course1
                        courselist3
                            course5
                            course6
                    course3
                    course4
                container5
            courselist1
                course1
                    courselist3
                        course5
                        course6
                course2
                    courselist3
                        course5
                        course6
                    courselist4
                        course3
                        course7
            subprogram3
                courselist2
                    course1
                        courselist3
                            course5
                            course6
                    course3
                    course4
                courselist3
                    course5
                    course6
                courselist4
                    course3
                    course7
        */
        /* Arborescence de pages
        program3
            subprogram1
                course1
                    course5
                    course6
                course3
                course4
            subprogram3
                course1
                    course5
                    course6
                course3
                course4
                course5
                course6
                course7
            course1
                course5
                course6
            course3
            course4
            course2
                course5
                course6
                course3
                course7
        */

        Page programPage = null;
        try
        {
            programPage = _ametysObjectResolver.resolveById("program://_root?rootId=" + page.getId() + "&programId=" + program3.getId());
        }
        catch (UnknownAmetysObjectException e)
        {
            Assert.fail("Failed to resolve virtual ODF page with id '" + "program://_root?rootId=" + page.getId() + "&programId=" + program3.getId() + "'.");
        }

        String rootId = "rootId=" + page.getId();
        String parentId = "&parentId=" + program3.getId();
        String programId = "&programId=" + program3.getId();
        
        // Contrôle de l'arborescence des pages de program3 (voir ci-dessus)
        Assert.assertNotNull(programPage);
        _assertPage(programPage, "formation-tu-3-[0-9A-Z]{8}", "program", rootId + programId, null, levelPath, 6); // Page de program3 avec 6 enfants et pas de parent
        _assertPage(programPage.getChildPageAt(0), "parcours-tu-1-[0-9A-Z]{8}", "program", rootId + "&programId=" + subProgram1.getId() + parentId, "formation-tu-3-[0-9A-Z]{8}", levelPath, 3); // Page de subProgram1 avec 3 enfants et enfant de program3
        _assertPage(programPage.getChildPageAt(0).getChildPageAt(0), "elp-tu-1-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course1.getId() + programId, "formation-tu-3-[0-9A-Z]{8}/parcours-tu-1-[0-9A-Z]{8}", levelPath, 2); // Page de course1 avec 2 enfants et enfant de program3/subProgram1
        _assertPage(programPage.getChildPageAt(0).getChildPageAt(0).getChildPageAt(0), "elp-tu-5-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course5.getId() + programId, "formation-tu-3-[0-9A-Z]{8}/parcours-tu-1-[0-9A-Z]{8}/elp-tu-1-[0-9A-Z]{8}", levelPath, 0); // Page de course5 sans enfant et enfant de program3/subProgram1/course1
        _assertPage(programPage.getChildPageAt(0).getChildPageAt(0).getChildPageAt(1), "elp-tu-6-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course6.getId() + programId, "formation-tu-3-[0-9A-Z]{8}/parcours-tu-1-[0-9A-Z]{8}/elp-tu-1-[0-9A-Z]{8}", levelPath, 0); // Page de course6 sans enfant et enfant de program3/subProgram1/course1
        _assertPage(programPage.getChildPageAt(0).getChildPageAt(1), "elp-tu-3-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course3.getId() + programId, "formation-tu-3-[0-9A-Z]{8}/parcours-tu-1-[0-9A-Z]{8}", levelPath, 0); // Page de course3 sans enfant et enfant de program3/subProgram1
        _assertPage(programPage.getChildPageAt(0).getChildPageAt(2), "elp-tu-4-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course4.getId() + programId, "formation-tu-3-[0-9A-Z]{8}/parcours-tu-1-[0-9A-Z]{8}", levelPath, 0); // Page de course4 sans enfant et enfant de program3/subProgram1
        _assertPage(programPage.getChildPageAt(1), "parcours-tu-3-[0-9A-Z]{8}", "program", rootId + "&programId=" + subProgram3.getId() + parentId, "formation-tu-3-[0-9A-Z]{8}", levelPath, 6); // Page de subProgram3 avec 6 enfants et enfant de program3
        _assertPage(programPage.getChildPageAt(1).getChildPageAt(0), "elp-tu-1-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course1.getId() + programId, "formation-tu-3-[0-9A-Z]{8}/parcours-tu-3-[0-9A-Z]{8}", levelPath, 2); // Page de course1 avec 2 enfants et enfant de program3/subProgram3
        _assertPage(programPage.getChildPageAt(1).getChildPageAt(0).getChildPageAt(0), "elp-tu-5-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course5.getId() + programId, "formation-tu-3-[0-9A-Z]{8}/parcours-tu-3-[0-9A-Z]{8}/elp-tu-1-[0-9A-Z]{8}", levelPath, 0); // Page de course5 sans enfant et enfant de program3/subProgram3/course1
        _assertPage(programPage.getChildPageAt(1).getChildPageAt(0).getChildPageAt(1), "elp-tu-6-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course6.getId() + programId, "formation-tu-3-[0-9A-Z]{8}/parcours-tu-3-[0-9A-Z]{8}/elp-tu-1-[0-9A-Z]{8}", levelPath, 0); // Page de course6 sans enfant et enfant de program3/subProgram3/course1
        _assertPage(programPage.getChildPageAt(1).getChildPageAt(1), "elp-tu-3-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course3.getId() + programId, "formation-tu-3-[0-9A-Z]{8}/parcours-tu-3-[0-9A-Z]{8}", levelPath, 0); // Page de course3 sans enfant et enfant de program3/subProgram3
        _assertPage(programPage.getChildPageAt(1).getChildPageAt(2), "elp-tu-4-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course4.getId() + programId, "formation-tu-3-[0-9A-Z]{8}/parcours-tu-3-[0-9A-Z]{8}", levelPath, 0); // Page de course4 sans enfant et enfant de program3/subProgram3
        _assertPage(programPage.getChildPageAt(1).getChildPageAt(3), "elp-tu-5-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course5.getId() + programId, "formation-tu-3-[0-9A-Z]{8}/parcours-tu-3-[0-9A-Z]{8}", levelPath, 0); // Page de course5 sans enfant et enfant de program3/subProgram3
        _assertPage(programPage.getChildPageAt(1).getChildPageAt(4), "elp-tu-6-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course6.getId() + programId, "formation-tu-3-[0-9A-Z]{8}/parcours-tu-3-[0-9A-Z]{8}", levelPath, 0); // Page de course6 sans enfant et enfant de program3/subProgram3
        _assertPage(programPage.getChildPageAt(1).getChildPageAt(5), "elp-tu-7-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course7.getId() + programId, "formation-tu-3-[0-9A-Z]{8}/parcours-tu-3-[0-9A-Z]{8}", levelPath, 0); // Page de course7 sans enfant et enfant de program3/subProgram3
        _assertPage(programPage.getChildPageAt(2), "elp-tu-1-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course1.getId() + programId, "formation-tu-3-[0-9A-Z]{8}", levelPath, 2); // Page de course1 avec 2 enfants et enfant de program3
        _assertPage(programPage.getChildPageAt(2).getChildPageAt(0), "elp-tu-5-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course5.getId() + programId, "formation-tu-3-[0-9A-Z]{8}/elp-tu-1-[0-9A-Z]{8}", levelPath, 0); // Page de course5 avec sans enfant et enfant de program3/course2
        _assertPage(programPage.getChildPageAt(2).getChildPageAt(1), "elp-tu-6-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course6.getId() + programId, "formation-tu-3-[0-9A-Z]{8}/elp-tu-1-[0-9A-Z]{8}", levelPath, 0); // Page de course6 avec sans enfant et enfant de program3/course2
        _assertPage(programPage.getChildPageAt(3), "elp-tu-3-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course3.getId() + programId, "formation-tu-3-[0-9A-Z]{8}", levelPath, 0); // Page de course3 avec sans enfant et enfant de program3
        _assertPage(programPage.getChildPageAt(4), "elp-tu-4-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course4.getId() + programId, "formation-tu-3-[0-9A-Z]{8}", levelPath, 0); // Page de course4 avec sans enfant et enfant de program3
        _assertPage(programPage.getChildPageAt(5), "elp-tu-2-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course2.getId() + programId, "formation-tu-3-[0-9A-Z]{8}", levelPath, 4); // Page de course2 avec 4 enfants et enfant de program3
        _assertPage(programPage.getChildPageAt(5).getChildPageAt(0), "elp-tu-5-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course5.getId() + programId, "formation-tu-3-[0-9A-Z]{8}/elp-tu-2-[0-9A-Z]{8}", levelPath, 0); // Page de course5 avec sans enfant et enfant de program3/course2
        _assertPage(programPage.getChildPageAt(5).getChildPageAt(1), "elp-tu-6-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course6.getId() + programId, "formation-tu-3-[0-9A-Z]{8}/elp-tu-2-[0-9A-Z]{8}", levelPath, 0); // Page de course6 avec sans enfant et enfant de program3/course2
        _assertPage(programPage.getChildPageAt(5).getChildPageAt(2), "elp-tu-3-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course3.getId() + programId, "formation-tu-3-[0-9A-Z]{8}/elp-tu-2-[0-9A-Z]{8}", levelPath, 0); // Page de course3 avec sans enfant et enfant de program3/course2
        _assertPage(programPage.getChildPageAt(5).getChildPageAt(3), "elp-tu-7-[0-9A-Z]{8}", "course", rootId + "&courseId=" + course7.getId() + programId, "formation-tu-3-[0-9A-Z]{8}/elp-tu-2-[0-9A-Z]{8}", levelPath, 0); // Page de course7 avec sans enfant et enfant de program3/course2
    }
    
    
    /**
     * Check some informations on a specific page.
     * @param page The page to control
     * @param pageName The excepted page name as a regex
     * @param scheme The scheme used by the page
     * @param idParameters The parameters to retrieve the page in the good context
     * @param path The expected page path as a regex
     * @param levelPath The previous path before the final page name
     * @param childrenSize The number of children pages
     */
    private void _assertPage(Page page, String pageName, String scheme, String idParameters, String path, String levelPath, long childrenSize)
    {
        // Check if the name of the page match the excepted page name regex
        _assertRegexp(page.getName(), pageName);
        
        // Check if the ID of th page match the expected page ID
        _assertRegexp(page.getId(), scheme + "://" + (path == null ? "_root" : path) + "\\?" + idParameters);
        
        // Check if the page path in the sitemap match the expected page path regex
        _assertRegexp(page.getPathInSitemap(), levelPath + (path == null ? "" : path + "/") + pageName);
        
        // Check if the page has the excepted number of children
        Assert.assertEquals(childrenSize, page.getChildrenPages().getSize());
    }
    
    /**
     * Improvment to test a regex on a string and display a readable message.
     * @param value The string to test
     * @param regex The regex to apply
     */
    private void _assertRegexp(String value, String regex)
    {
        Assert.assertTrue("The value '" + value + "' doesn't match '^" + regex + "$'.", value.matches("^" + regex + "$"));
    }
}
