/*
 *  Copyright 2019 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.plugins.odfweb;

import java.io.FileInputStream;
import java.io.InputStream;
import java.time.ZonedDateTime;

import javax.xml.parsers.SAXParserFactory;

import org.apache.avalon.framework.configuration.Configuration;
import org.apache.avalon.framework.configuration.DefaultConfigurationBuilder;
import org.xml.sax.XMLReader;

import org.ametys.cms.repository.ModifiableDefaultContent;
import org.ametys.core.user.UserIdentity;
import org.ametys.plugins.repository.AmetysObjectResolver;
import org.ametys.plugins.repository.ModifiableTraversableAmetysObject;
import org.ametys.plugins.repository.data.holder.ModifiableModelAwareDataHolder;
import org.ametys.plugins.repository.root.RootAmetysObject;
import org.ametys.runtime.model.type.ElementType;
import org.ametys.web.repository.page.ModifiablePage;
import org.ametys.web.repository.page.ModifiableZone;
import org.ametys.web.repository.page.ModifiableZoneItem;
import org.ametys.web.repository.page.Page.LinkType;
import org.ametys.web.repository.page.Page.PageType;
import org.ametys.web.repository.page.SitemapElement;
import org.ametys.web.repository.page.ZoneItem.ZoneType;
import org.ametys.web.repository.site.Site;
import org.ametys.web.repository.site.SiteManager;
import org.ametys.web.repository.site.SiteType;
import org.ametys.web.repository.sitemap.Sitemap;

/**
 * Sitemap configurator
 */
public class SitemapConfigurator
{
    /** The nodetype for default content */
    public static final String NT_DEFAULT_CONTENT = "ametys:defaultContent";
    /** The nodetype for default page */
    public static final String NT_DEFAULT_PAGE = "ametys:defaultPage";
    
    private AmetysObjectResolver _ametysObjectResolver;
    private SiteManager _siteManager;
    
    private UserIdentity _creator;
    
    /**
     * Constructor
     * @param ametysObjectResolver The resolver
     * @param siteManager The site manager
     * @param creator The user which will be set as creator and last contributor on created contents
     */
    public SitemapConfigurator(AmetysObjectResolver ametysObjectResolver, SiteManager siteManager, UserIdentity creator)
    {
        _ametysObjectResolver = ametysObjectResolver;
        _siteManager = siteManager;
        _creator = creator;
    }
    
    /**
     * Configures sites and sitemaps from the given file
     * @param fileName the name of the sitemap configuration file
     * @throws Exception if the sitemaps cannot be created
     */
    protected void _configureSitemaps(String fileName) throws Exception
    {
        SAXParserFactory factory = SAXParserFactory.newInstance();
        XMLReader reader = factory.newSAXParser().getXMLReader();
        
        DefaultConfigurationBuilder confBuilder = new DefaultConfigurationBuilder(reader);
        
        InputStream is = new FileInputStream(fileName);
        
        Configuration conf = confBuilder.build(is, fileName);
        
        Configuration[] sitesConf = conf.getChildren("site");
        for (Configuration siteConf : sitesConf)
        {
            Site site = _siteManager.createSite(siteConf.getAttribute("name"), null);
            site.setType(siteConf.getAttribute("type", SiteType.DEFAULT_SITE_TYPE_ID));
            
            Configuration[] sitemapsConf = siteConf.getChildren("sitemap");
            for (Configuration sitemapConf : sitemapsConf)
            {
                Sitemap sitemap = site.addSitemap(sitemapConf.getAttribute("name"));
                _configurePages(sitemap, sitemapConf);
            }
            site.saveChanges();
        }
    }
    
    /**
     * Configures pages
     * @param sitemapElement the parent sitemap or page
     * @param configuration the pages configuration
     * @throws Exception if pages can not be created
     */
    protected void _configurePages(SitemapElement sitemapElement, Configuration configuration) throws Exception
    {
        for (Configuration pageConf : configuration.getChildren("page"))
        {
            ModifiablePage page = ((ModifiableTraversableAmetysObject) sitemapElement).createChild(pageConf.getAttribute("name"), NT_DEFAULT_PAGE);
            page.setTitle(pageConf.getAttribute("title"));
            page.setSiteName(sitemapElement.getSiteName());
            page.setSitemapName(sitemapElement.getSitemapName());
            
            PageType pageType = PageType.valueOf(pageConf.getAttribute("type", "NODE"));
            page.setType(pageType);
            
            switch (pageType)
            {
                case LINK:
                    Configuration urlConf = pageConf.getChild("url");
                    LinkType linkType = LinkType.valueOf(urlConf.getAttribute("type", "WEB"));
                    page.setURL(linkType, urlConf.getValue());
                    break;

                case CONTAINER:
                    page.setTemplate(pageConf.getAttribute("template", "page"));
                    _configureZones(page, pageConf);
                    break;
                    
                default:
                    break;
            }
            
            _configurePages(page, pageConf);
        }
    }
    
    /**
     * Configures and creates zones
     * @param page the parent page
     * @param configuration the zones configuration 
     * @throws Exception if zones can not be created
     */
    protected void _configureZones(ModifiablePage page, Configuration configuration) throws Exception
    {
        for (Configuration zoneConf : configuration.getChildren("zone"))
        {
            ModifiableZone zone = page.createZone(zoneConf.getAttribute("name"));
            
            for (Configuration childConf : zoneConf.getChildren())
            {
                if ("service".equals(childConf.getName()))
                {
                    ModifiableZoneItem zoneItem = zone.addZoneItem();
                    zoneItem.setType(ZoneType.SERVICE);
                    zoneItem.setServiceId(childConf.getAttribute("id"));
                    
                    ModifiableModelAwareDataHolder serviceDataHolder = zoneItem.getServiceParameters();
                    
                    for (Configuration paramConf : childConf.getChildren("param"))
                    {
                        String parameterName = paramConf.getAttribute("name");
                        String valuesAsString = paramConf.getValue();
                        ElementType type = (ElementType) serviceDataHolder.getType(parameterName);
                        serviceDataHolder.setValue(parameterName, type.castValue(valuesAsString));
                    }
                }
                else if ("content".equals(childConf.getName()))
                {
                    ModifiableZoneItem zoneItem = zone.addZoneItem();
                    zoneItem.setType(ZoneType.CONTENT);
                    _configureContent(zoneItem, childConf, page.getSitemapName());
                }
                
            }
        }
    }
    
    /**
     * Configures and creates a content 
     * @param zoneItem the parent zone item
     * @param configuration the content configuration
     * @param lang the content language
     * @throws Exception if content can not be created
     */
    protected void _configureContent(ModifiableZoneItem zoneItem, Configuration configuration, String lang) throws Exception
    {
        RootAmetysObject root = _ametysObjectResolver.resolveByPath("/");
        ModifiableDefaultContent defaultContent = root.createChild(configuration.getAttribute("name"), NT_DEFAULT_CONTENT);
        
        defaultContent.setTypes(new String[]{configuration.getAttribute("type")});
        defaultContent.setLanguage(lang);
        defaultContent.setTitle(configuration.getAttribute("title"));
        defaultContent.setCreationDate(ZonedDateTime.now());
        defaultContent.setCreator(_creator);
        defaultContent.setLastContributor(_creator);
        defaultContent.setLastModified(ZonedDateTime.now());
    }
}
